datablock AudioProfile(LightClimbSound)
{
   filename = "fx/armor/light_land_hard.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(MediumClimbSound)
{
   filename = "fx/armor/med_land_hard.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(HeavyClimbSound)
{
   filename = "fx/armor/heavy_land_hard.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(WallJumpSound)
{
   filename = "fx/powered/turret_heavy_reload.wav";
   description = AudioClose3d;
   preload = true;
};

function Player::findValidFrontEyedWall(%player)
{
   %searchRange = 1.75;

   // We want the vector to be directly where the player is looking 
   // so do nothing to the eyevec.
   %eyeVec   = %player.getEyeVector();
   %eyePos = posFromTransform(%player.getEyeTransform());

   %nEyeVec = VectorNormalize(%eyeVec);

   %scEyeVec = VectorScale(%nEyeVec, %searchRange);

   %eyeEnd = VectorAdd(%eyePos, %scEyeVec);

   %searchResult = containerRayCast(%eyePos, %eyeEnd, $TypeMasks::InteriorObjectType, 0);
   %searchNormal = normalFromRaycast(%searchResult);
   %searchAngle = getTerrainAngle(%searchNormal);
   if(isObject(getWord(%searchResult,0)) && (mFloor(%searchAngle) >= 75 && mFloor(%searchAngle) <= 105))
      return true;
   else
      return false;
}

function Player::findValidFrontWall(%player)
{
   %searchRange = 1.75;

   // We want the vector to be directly in front of the player, so we
   // remove the Z axis from the vector so its right in the middle.
   %eyeVec   = getWord(%player.getEyeVector(),0) SPC getWord(%player.getEyeVector(),1) SPC "0";
   %eyePos = %player.getWorldBoxCenter();

   %nEyeVec = VectorNormalize(%eyeVec);

   %scEyeVec = VectorScale(%nEyeVec, %searchRange);

   %eyeEnd = VectorAdd(%eyePos, %scEyeVec);

   %searchResult = containerRayCast(%eyePos, %eyeEnd, $TypeMasks::InteriorObjectType, 0);
   %searchNormal = normalFromRaycast(%searchResult);
   %searchAngle = getTerrainAngle(%searchNormal);
   if(isObject(getWord(%searchResult,0)) && (mFloor(%searchAngle) >= 75 && mFloor(%searchAngle) <= 105))
      return true;
   else
      return false;
}

function Player::applyClimbWall(%player)
{
   // Search for if they are looking at the wall first, then search
   // for if they are looking somewhere else (Z vec) but they are facing the wall.
   if(%player.findValidFrontEyedWall())
   {
      %player.playAudio(0, %player.getArmorSize() @ "ClimbSound");
      %player.applyImpulse(%player.getTransform(), "0" SPC "0" SPC %player.getDataBlock().jumpForce);
      %player.setActionThread("jump");
   }
   else if(%player.findValidFrontWall())
   {
      %player.playAudio(0, %player.getArmorSize() @ "ClimbSound");
      %player.applyImpulse(%player.getTransform(), "0" SPC "0" SPC %player.getDataBlock().jumpForce);
      %player.setActionThread("jump");
   }
}

function Player::findValidRearEyedWall(%player)
{
   %searchRange = 1.75;

   // We want the vector to be directly to the rear of where the player is 
   // looking, so we just change a negative to positive or positive to negative.
   %eyeVec   = (getWord(%player.getEyeVector(),0) - (getWord(%player.getEyeVector(),0) * 2))  SPC (getWord(%player.getEyeVector(),1) - (getWord(%player.getEyeVector(),1) * 2)) SPC (getWord(%player.getEyeVector(),2) - (getWord(%player.getEyeVector(),2) * 2));
   // We still want it to be in the center becuase the player is jumping,
   // not climbing... we don't use our hands to jump off a wall, only for
   // climbing.
   %eyePos = %player.getWorldBoxCenter();

   %nEyeVec = VectorNormalize(%eyeVec);

   %scEyeVec = VectorScale(%nEyeVec, %searchRange);

   %eyeEnd = VectorAdd(%eyePos, %scEyeVec);

   %searchResult = containerRayCast(%eyePos, %eyeEnd, $TypeMasks::InteriorObjectType, 0);
   %searchNormal = normalFromRaycast(%searchResult);
   %searchAngle = getTerrainAngle(%searchNormal);
   if(isObject(getWord(%searchResult,0)) && (mFloor(%searchAngle) >= 75 && mFloor(%searchAngle) <= 105))
      return true;
   else
      return false;
}

function Player::findValidRearWall(%player)
{
   %searchRange = 1.75;

   // We want the vector to be directly to the rear of the player, so we
   // remove the Z axis from the vector so its right in the middle and then
   // change a negative to positive or positive to negative.
   %eyeVec   = (getWord(%player.getEyeVector(),0) - (getWord(%player.getEyeVector(),0) * 2))  SPC (getWord(%player.getEyeVector(),1) - (getWord(%player.getEyeVector(),1) * 2)) SPC "0";
   %eyePos = %player.getWorldBoxCenter();

   %nEyeVec = VectorNormalize(%eyeVec);

   %scEyeVec = VectorScale(%nEyeVec, %searchRange);

   %eyeEnd = VectorAdd(%eyePos, %scEyeVec);

   %searchResult = containerRayCast(%eyePos, %eyeEnd, $TypeMasks::InteriorObjectType, 0);
   %searchNormal = normalFromRaycast(%searchResult);
   %searchAngle = getTerrainAngle(%searchNormal);
   if(isObject(getWord(%searchResult,0)) && (mFloor(%searchAngle) >= 75 && mFloor(%searchAngle) <= 105))
      return true;
   else
      return false;
}

function Player::applyJumpWall(%player)
{
   // Search for if they are looking at the opposite direction of the wall
   // first, then search for if they are looking somewhere else (Z vec) but 
   // the player's back is facing the wall.
   if(%player.findValidRearEyedWall())
   {
      %impulseStrength = 2500;
      %impulseAmount = VectorScale(VectorNormalize(%player.getEyeVector()), %impulseStrength);

      %player.playAudio(0, %player.getArmorSize() @ "ClimbSound");
      %player.schedule(75, playAudio, 0, "WallJumpSound");
      %player.applyImpulse(%player.getTransform(), %impulseAmount);
      %player.setActionThread("jet");
   }
   else if(%player.findValidRearWall())
   {
      %impulseStrength = 2500;
      %impulseAmount = VectorScale(VectorNormalize(%player.getEyeVector()), %impulseStrength);

      %player.playAudio(0, %player.getArmorSize() @ "ClimbSound");
      %player.schedule(75, playAudio, 0, "WallJumpSound");
      %player.applyImpulse(%player.getTransform(), %impulseAmount);
      %player.setActionThread("jet");
   }
}

