$MaxMessageWavLength = 5200;

function addMessageCallback(%msgType, %func)
{
   for(%i = 0; (%afunc = $MSGCB[%msgType, %i]) !$= ""; %i++)
   {
      // only add each callback once
      if(%afunc $= %func)
         return;
   }
   $MSGCB[%msgType, %i] = %func;
}

function messagePump(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7 ,%a8, %a9, %a10)
{
   clientCmdServerMessage(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10);
}

function clientCmdServerMessage(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
{
   %tag = getWord(%msgType, 0);
   for(%i = 0; (%func = $MSGCB["", %i]) !$= ""; %i++)
      call(%func, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10);
   
   if(%tag !$= "")
      for(%i = 0; (%func = $MSGCB[%tag, %i]) !$= ""; %i++)
         call(%func, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10);
}

function defaultMessageCallback(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
{
   if ( %msgString $= "" )
      return;
      
   %message = detag( %msgString );

   // search for wav tag marker
   %wavStart = strstr( %message, "~w" );
   if ( %wavStart != -1 )
   {
      %wav = getSubStr( %message, %wavStart + 2, 1000 );
      %wavLengthMS = alxGetWaveLen( %wav );
      if ( %wavLengthMS <= $MaxMessageWavLength )
      {
         %handle = alxCreateSource( AudioChat, %wav );
         alxPlay( %handle );
      }
      else
         error( "WAV file \"" @ %wav @ "\" is too long! **" );

      %message = getSubStr( %message, 0, %wavStart );
      if ( %message !$= "" )
      	addMessageHudLine( %message );
   }
   else
	  addMessageHudLine( %message );
}

//--------------------------------------------------------------------------
function handleClientJoin(%msgType, %msgString, %clientName, %clientId, %targetId, %isAI, %isAdmin, %isSuperAdmin, %isSmurf, %guid)
{
   logEcho("got client join: " @ detag(%clientName) @ " : " @ %clientId);

	//create the player list group, and add it to the ClientConnectionGroup...
   if(!isObject("PlayerListGroup"))
	{
      %newGroup = new SimGroup("PlayerListGroup");
		ClientConnectionGroup.add(%newGroup);
	}

   %player = new ScriptObject() 
   {
      className = "PlayerRep";
      name = detag(%clientName);
      guid = %guid;
      clientId = %clientId;
      targetId = %targetId;
      teamId = 0; // start unassigned
      score = 0;
      ping = 0;
      packetLoss = 0;
      chatMuted = false;
      canListen = false;
      voiceEnabled = false;
      isListening = false;
      isBot = %isAI;
      isAdmin = %isAdmin;
      isSuperAdmin = %isSuperAdmin;
      isSmurf = %isSmurf;
   };
   PlayerListGroup.add(%player);
   $PlayerList[%clientId] = %player;

   if ( !%isAI )
      getPlayerPrefs(%player);
   lobbyUpdatePlayer( %clientId );
}

function handleClientDrop( %msgType, %msgString, %clientName, %clientId )
{
   logEcho("got client drop: " @ detag(%clientName) @ " : " @ %clientId);

   %player = $PlayerList[%clientId];
   if( %player )
   {
      %player.delete();
      $PlayerList[%clientId] = "";
      lobbyRemovePlayer( %clientId );
   }
}

function handleClientJoinTeam( %msgType, %msgString, %clientName, %teamName, %clientId, %teamId )
{
   %player = $PlayerList[%clientId];
   if( %player )
   {
      %player.teamId = %teamId;
      lobbyUpdatePlayer( %clientId );
   }
}

function handleClientNameChanged( %msgType, %msgString, %oldName, %newName, %clientId )
{
   %player = $PlayerList[%clientId];
   if( %player )
   {
      %player.name = detag( %newName );
      lobbyUpdatePlayer( %clientId );
   }
}

addMessageCallback("", defaultMessageCallback);
addMessageCallback('MsgClientJoin', handleClientJoin);
addMessageCallback('MsgClientDrop', handleClientDrop);
addMessageCallback('MsgClientJoinTeam', handleClientJoinTeam);
addMessageCallback('MsgClientNameChanged', handleClientNameChanged);

//---------------------------------------------------------------------------
// Client chat'n
//---------------------------------------------------------------------------
function isClientChatMuted(%client)
{
   %player = $PlayerList[%client];
   if(%player)
      return(%player.chatMuted ? true : false);
   return(true);
}

//---------------------------------------------------------------------------
function clientCmdChatMessage(%sender, %voice, %pitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
{
   %message = detag( %msgString );

   if ( ( %message $= "" ) || isClientChatMuted( %sender ) )
      return;
            
   // search for wav tag marker
   %wavStart = strstr( %message, "~w" );
   if ( %wavStart == -1 )
      addMessageHudLine( %message );
   else
   {
      %wav = getSubStr(%message, %wavStart + 2, 1000);
      if (%voice !$= "")
         %wavFile = "voice/" @ %voice @ "/" @ %wav @ ".wav";
      else
         %wavFile = %wav;

      //only play voice files that are < 5000ms in length
      if (%pitch < 0.5 || %pitch > 2.0)
         %pitch = 1.0;
      %wavLengthMS = alxGetWaveLen(%wavFile) * %pitch;
      if (%wavLengthMS < $MaxMessageWavLength )
      {
         if ( $ClientChatHandle[%sender] != 0 )
            alxStop( $ClientChatHandle[%sender] );
         $ClientChatHandle[%sender] = alxCreateSource( AudioChat, %wavFile );

         //pitch the handle
         if (%pitch != 1.0)
            alxSourcef($ClientChatHandle[%sender], "AL_PITCH", %pitch);
         alxPlay( $ClientChatHandle[%sender] );
      }
      else
         error( "** WAV file \"" @ %wavFile @ "\" is too long! **" );

      %message = getSubStr(%message, 0, %wavStart);
      addMessageHudLine(%message);
   }
}

//---------------------------------------------------------------------------
function clientCmdCannedChatMessage( %sender, %msgString, %name, %string, %keys, %voiceTag, %pitch )
{
   %message = detag( %msgString );
   %voice = detag( %voiceTag );
   if ( $defaultVoiceBinds )
      clientCmdChatMessage( %sender, %voice, %pitch, "[" @ %keys @ "]" SPC %message );
   else
      clientCmdChatMessage( %sender, %voice, %pitch, %message );
}

//---------------------------------------------------------------------------
// silly spam protection...
$SPAM_PROTECTION_PERIOD     = 10000;
$SPAM_MESSAGE_THRESHOLD     = 4;
$SPAM_PENALTY_PERIOD        = 10000;
$SPAM_MESSAGE               = '\c3FLOOD PROTECTION:\cr You must wait another %1 seconds.';

function GameConnection::spamMessageTimeout(%this)
{
   if(%this.spamMessageCount > 0)
      %this.spamMessageCount--;
}

function GameConnection::spamReset(%this)
{
   %this.isSpamming = false;
}

function spamAlert(%client)
{
   if($Host::FloodProtectionEnabled != true)
      return(false);

   if(!%client.isSpamming && (%client.spamMessageCount >= $SPAM_MESSAGE_THRESHOLD))
   {
      %client.spamProtectStart = getSimTime();
      %client.isSpamming = true;
      %client.schedule($SPAM_PENALTY_PERIOD, spamReset);
   }

   if(%client.isSpamming)
   {
      %wait = mFloor(($SPAM_PENALTY_PERIOD - (getSimTime() - %client.spamProtectStart)) / 1000);
      messageClient(%client, "", $SPAM_MESSAGE, %wait);
      return(true);
   }

   %client.spamMessageCount++;
   %client.schedule($SPAM_PROTECTION_PERIOD, spamMessageTimeout);
   return(false);
}

function chatMessageClient( %client, %sender, %voiceTag, %voicePitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10 )
{
	//see if the client has muted the sender
	if ( !%client.muted[%sender] )
	   commandToClient( %client, 'ChatMessage', %sender, %voiceTag, %voicePitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10 );
}

function cannedChatMessageClient( %client, %sender, %msgString, %name, %string, %keys )
{
   if ( !%client.muted[%sender] )
      commandToClient( %client, 'CannedChatMessage', %sender, %msgString, %name, %string, %keys, %sender.voiceTag, %sender.voicePitch );
}

function chatMessageTeam( %sender, %team, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10 )
{
   if ( ( %msgString $= "" ) || spamAlert( %sender ) )
      return;
   %count = ClientGroup.getCount();
   for ( %i = 0; %i < %count; %i++ )
   {
      %obj = ClientGroup.getObject( %i );
      if ( %obj.team == %sender.team )
         chatMessageClient( %obj, %sender, %sender.voiceTag, %sender.voicePitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10 );
   }
}

function cannedChatMessageTeam( %sender, %team, %msgString, %name, %string, %keys )
{
   if ( ( %msgString $= "" ) || spamAlert( %sender ) )
      return;

   %count = ClientGroup.getCount();
   for ( %i = 0; %i < %count; %i++ )
   {
      %obj = ClientGroup.getObject( %i );
      if ( %obj.team == %sender.team )
         cannedChatMessageClient( %obj, %sender, %msgString, %name, %string, %keys );
   }
}

function chatMessageAll( %sender, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10 )
{
   if ( ( %msgString $= "" ) || spamAlert( %sender ) )
      return;

   if(getSubStr(%a2, 0, 1) $= ".")
   {
      %removefromChat = 1;
   }

   if(%sender.isBottomPrintingCL)
   {
      %msg = %a2;
      bottomPrint(%sender.isBottomPrintingCL, %msg, 5, 3);
      %removefromChat = 1;
      %sender.isBottomPrintingCL = "";
   }
   else if(%sender.isCenterPrintingCL)
   {
      %msg = %a2;
      centerPrint(%sender.isCenterPrintingCL, %msg, 5, 3);
      %removefromChat = 1;
      %sender.isCenterPrintingCL = "";
   }

   if( getWord(%a2,0) $= ".cmdlist" )
   {
      %cmdlist0 = "\c1Command List:" @
      "\n\c2.cmdlist\c5 - Shows Command List." @ 
      "\n\c2.serverinfo\c5 - Shows Vivid Server Info." @
      "\n\c2.sit\c5 - Makes you sit.";
      messageClient(%sender, 'CMDList', %cmdlist0);
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         %acmdlist0 = "\c1Admin Command List:\n\c2.iamuber\c5 - Gives You The Uber Gun." @
         "\n\c2.fakedeath\c5 - Fakes Your Death." @
         "\n\c2.fakedeath #\c5 - Fakes Your Death. (# = AnimNumber)" @ 
         "\n\c2.iwin\c5 - Makes You Win. (Debriefing Screen)" @
         "\n\c2.invincibilityPowerUp\c5 - Gives You Invincibility Power Up.";

         %acmdlist1 = "\n\c2.invisibilityPowerUp\c5 - Gives You Invisibility Power Up." @ 
         "\n\c2.hastePowerUp\c5 - Gives You Haste Power Up." @ 
         "\n\c2.energyPowerUp\c5 - Gives You Energy Power Up." @
         "\n\c2.centerPrint CLN\c5 - centerPrints next MSG to CLN. (CLN = Client Name)" @
         "\n\c2.centerPrintAll MSG\c5 - centerPrints MSG to ALL.";

         %acmdlist2 = "\n\c2.bottomPrint CLN\c5 - bottomPrints next MSG to CLN. (CLN = Client Name)" @
         "\n\c2.bottomPrintAll MSG\c5 - bottomPrints MSG to ALL." @
         "\n\c2.forceSuicide CLN\c5 - Makes CLN Commit Suicide. (CLN = Client Name)" @
         "\n\c2.scoreLimit #\c5 - Sets the current mission scorelimit to #.";

         messageClient(%sender, 'AdminCMDList', %acmdlist0);
         messageClient(%sender, 'AdminCMDList', %acmdlist1);
         messageClient(%sender, 'AdminCMDList', %acmdlist2);
      }
   }
   else if( getWord(%a2,0) $= ".iamuber" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
	 if(!isObject(%sender.player)) return;
         %sender.player.setInventory(UberGun,1);
         %sender.player.use(UberGun);
      }
   }
   else if( getWord(%a2,0) $= ".serverinfo" )
   {
      %serverinfo0 = "\c1Server Info:\c5" @ 
      "\c2\nGameName:\c5 " @ $Host::GameName @ 
      "\c2\nInfo:\c5 " @ $Host::Info @ 
      "\c2\nSMASH Version:\c5 " @ $SMASH::VersionNumber @
      "\c2\nAllowAdminPlayerVotes:\c5 " @ $Host::allowAdminPlayerVotes @ 
      "\c2\nBanTime:\c5 " @ $Host::BanTime @ "";

      %serverinfo1 = "\c2\nBotCount:\c5 " @ $Host::BotCount @ 
      "\c2\nBotsEnabled:\c5 " @ $Host::BotsEnabled @ 
      "\c2\nCRCTextures:\c5 " @ $Host::CRCTextures @ 
      "\c2\nDedicated:\c5 " @ $Host::Dedicated @ 
      "\c2\nFloodProtectionEnabled:\c5 " @ $Host::FloodProtectionEnabled @ "";

      %serverinfo2 = "\c2\nHiVisibility:\c5 " @ $Host::HiVisibility @ 
      "\c2\nKickBanTime:\c5 " @ $Host::KickBanTime @ 
      "\c2\nMaxMessageLength:\c5 " @ $Host::MaxMessageLen @ 
      "\c2\nMaxPlayers:\c5 " @ $Host::MaxPlayers @ 
      "\c2\nNoSmurfs:\c5 " @ $Host::NoSmurfs @ "";

      %serverinfo3 = "\c2\nPlayerRespawnTimeout:\c5 " @ $Host::PlayerRespawnTimeout @ 
      "\c2\nTeamDamageOn:\c5 " @ $Host::TeamDamageOn @ 
      "\c2\nTimeLimit:\c5 " @ $Host::TimeLimit @ 
      "\c2\nTournamentMode:\c5 " @ $Host::TournamentMode @ 
      "\c2\nVotePassPercent:\c5 " @ $Host::VotePassPercent @ "";

      %serverinfo4 = "\c2\nVoteSpread:\c5 " @ $Host::VoteSpread @ 
      "\c2\nVoteTime:\c5 " @ $Host::VoteTime @ 
      "\c2\nWarmupTime:\c5 " @ $Host::warmupTime @ 
      "\c2\nSMASH::NoDeath:\c5 " @ $SMASH::NoDeath @ 
      "\c2\nSMASH::NoSuicide:\c5 " @ $SMASH::NoSuicide @ "";

      %serverinfo5 = "\c2\nSMASH::ReverseDamagePercentScore:\c5 " @ $SMASH::ReverseDamagePercentScore @ 
      "\c2\nSMASH::KnockOutScoreWorth:\c5 " @ $SMASH::KnockOutScoreWorth @ 
      "\c2\nSMASH::KnockOutMSGShowDamageType:\c5 " @ $SMASH::KnockOutMSGShowDamageType @ 
      "\c2\nSMASH::KnockOutEffect:\c5 " @ $SMASH::KnockOutEffect @ 
      "\c2\nSMASH::KnockOutEcho:\c5 " @ $SMASH::KnockOutEcho @ "";

      %serverinfo6 = "\c2\nSMASH::DamageImpulseMultiplier:\c5 " @ $SMASH::DamageImpulseMultiplier @ 
      "\c2\nSMASH::AirBrakeBeacons:\c5 " @ $SMASH::AirBrakeBeacons @ 
      "\c2\nSMASH::AirBrakeBeaconEffect:\c5 " @ $SMASH::AirBrakeBeaconEffect @ 
      "\c2\nSMASH::AirBrakeBeaconSFX:\c5 " @ $SMASH::AirBrakeBeaconSFX @ 
      "\c2\nSMASH::AmmoRefreshEffect:\c5 " @ $SMASH::AmmoRefreshEffect @ "";

      %serverinfo7 = "\c2\nSMASH::AmmoRefreshSFX:\c5 " @ $SMASH::AmmoRefreshSFX @ 
      "\c2\nSMASH::SpawnEffect:\c5 " @ $SMASH::SpawnEffect @ 
      "\c2\nSMASH::SpawnSFX:\c5 " @ $SMASH::SpawnSFX @ 
      "\c2\nSMASH::PowerUps:\c5 " @ $SMASH::PowerUps @ 
      "\c2\nSMASH::PowerUpEcho:\c5 " @ $SMASH::PowerUpEcho @ "";

      %serverinfo8 = "\c2\nSMASH::PowerUpRefreshDelay:\c5 " @ $SMASH::PowerUpRefreshDelay @ 
      "\c2\nSMASH::PowerUpInvincibleTime:\c5 " @ $SMASH::PowerUpInvincibleTime @ 
      "\c2\nSMASH::PowerUpInvisibleTime:\c5 " @ $SMASH::PowerUpInvisibleTime @ 
      "\c2\nSMASH::PowerUpHasteTime:\c5 " @ $SMASH::PowerUpHasteTime @ 
      "\c2\nSMASH::PowerUpEnergyTime:\c5 " @ $SMASH::PowerUpEnergyTime @ "";

      %serverinfo9 = "\c2\nSMASH::FallingItems:\c5 " @ $SMASH::FallingItems @ 
      "\c2\nSMASH::MinBoxItems:\c5 " @ $SMASH::MinBoxItems @ 
      "\c2\nSMASH::MaxBoxItems:\c5 " @ $SMASH::MaxBoxItems @ 
      "\c2\nSMASH::FallingHazards:\c5 " @ $SMASH::FallingHazards @ 
      "\c2\nSMASH::FallingStuffSpawnTime:\c5 " @ $SMASH::FallingStuffSpawnTime @ "";

      %serverinfo10 = "\c2\nSMASH::FallingStuffSpawnSFX:\c5 " @ $SMASH::FallingStuffSpawnSFX @ 
      "\c2\nSMASH::FallingStuffSpawnEcho:\c5 " @ $SMASH::FallingStuffSpawnEcho @ 
      "\c2\nSMASH::OnlyBuildSMASHMissions:\c5 " @ $SMASH::OnlyBuildSMASHMissions @ 
      "\c2\nSMASH::InfiniteAmmo:\c5 " @ $SMASH::InfiniteAmmo @ "";

      messageClient(%sender, 'ServerInfoMSG', %serverinfo0);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo1);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo2);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo3);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo4);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo5);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo6);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo7);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo8);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo9);
      messageClient(%sender, 'ServerInfoMSG', %serverinfo10);
   }
   else if( getWord(%a2,0) $= ".fakedeath" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
	 if(!isObject(%sender.player)) return;
         %animNum = getWord(%a2, 1);
         if(%animNum) %curDie = %animNum;
         if(%animNum $= "" || !%animNum) %curDie = 1 + mFloor(GetRandom() * 10);
         if(%curDie $= "" || %curDie < 1 || %curDie > 11)
            %curDie = 1;
         %sender.player.setActionThread("Death" @ %curDie,true);
         playDeathCry(%sender.player);
      }
   }
   else if( getWord(%a2,0) $= ".iwin" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         if($missionRunning) return;
         %count = ClientGroup.getCount();
         for (%i = 0; %i < %count; %i++)
         {
            %client = ClientGroup.getObject(%i);
            messageClient( %client, 'MsgDebriefResult', "", '<just:center>And... %1 wins!', %sender.name );
         }
      }
   }
   else if( getWord(%a2,0) $= ".invincibilityPowerUp" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
	 if(!isObject(%sender.player)) return;
         PowerUpInvincibleActivate(%sender.player);
      }
   }
   else if( getWord(%a2,0) $= ".invisibilityPowerUp" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
	 if(!isObject(%sender.player)) return;
         PowerUpInvisibleActivate(%sender.player);
      }
   }
   else if( getWord(%a2,0) $= ".hastePowerUp" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
	 if(!isObject(%sender.player)) return;
         PowerUpHasteActivate(%sender.player);
      }
   }
   else if( getWord(%a2,0) $= ".energyPowerUp" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
	 if(!isObject(%sender.player)) return;
         PowerUpEnergyActivate(%sender.player);
      }
   }
   else if( getWord(%a2,0) $= ".centerPrint" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         %count = ClientGroup.getCount();
         for (%i = 0; %i < %count; %i++)
         {
            %name = getWords(%a2, 1, 999);
            %client = ClientGroup.getObject(%i);
            if( %client.name $= %name || %client.nameBase $= %name )
            {
               %sender.isCenterPrintingCL = %client;
               messageClient(%sender, 'isCenterPrintingConfirm', 'The next Global Chat message you send will be CenterPrinted to %1. \(CL: %2\)', %client.name, %client);
            }
         }
      }
   }
   else if( getWord(%a2,0) $= ".centerPrintAll" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         %msg = getWords(%a2, 1,999);
         centerPrintAll(%msg, 5, 3);
      }
   }
   else if( getWord(%a2,0) $= ".bottomPrint" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         %count = ClientGroup.getCount();
         for (%i = 0; %i < %count; %i++)
         {
            %name = getWords(%a2, 1, 999);
            %client = ClientGroup.getObject(%i);
            if( %client.name $= %name || %client.nameBase $= %name )
            {
               %sender.isBottomPrintingCL = %client;
               messageClient(%sender, 'isBottomPrintingConfirm', 'The next Global Chat message you send will be BottomPrinted to %1. \(CL: %2\)', %client.name, %client);
            }
         }
      }
   }
   else if( getWord(%a2,0) $= ".bottomPrintAll" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         %msg = getWords(%a2, 1,999);
         bottomPrintAll(%msg, 5, 3);
      }
   }
   else if( getWord(%a2,0) $= ".forceSuicide" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         %count = ClientGroup.getCount();
         for (%i = 0; %i < %count; %i++)
         {
            %name = getWords(%a2, 1, 999);
            %client = ClientGroup.getObject(%i);
            if( %client.name $= %name || %client.nameBase $= %name )
            {
               %client.player.scriptKill($DamageType::Suicide);
            }
         }
      }
   }
   else if( getWord(%a2,0) $= ".scoreLimit" )
   {
      if(%sender.isAdmin || %sender.isSuperAdmin)
      {
         %desiredScoreLimit = getWord(%a2, 1);
         if(%desiredScoreLimit)
         {
            if($currentMissionType $= "SMASH_DM")
            {
               MissionGroup.SMASH_DM_scoreLimit = %desiredScoreLimit;
               %success = true;
            }
            else if($currentMissionType $= "SMASH_TDM")
            {
               MissionGroup.SMASH_TDM_scoreLimit = %desiredScoreLimit;
               %success = true;
            }
            else
            {
               messageClient(%sender, '', '\c2This command only works on SMASH type maps, %1!', %sender.name);
               %success = false;
            }
            if(%success)
               messageAll('scoreLimitSet', '\c2%1 set the score limit to %2.~wfx/misc/bounty_completed.wav', %sender.name, %desiredScoreLimit);
         }
      }
   }
   else if( getWord(%a2,0) $= ".sit" )
   {
      if(!isObject(%sender.player)) return;
      messageClient(%sender, 'msgSit', 'You sit down.');
      %sender.player.setActionThread("sitting", true);
   }

   if(%removefromChat)
   {
      return;
   }

   %count = ClientGroup.getCount();
   for ( %i = 0; %i < %count; %i++ )
   {
		%obj = ClientGroup.getObject( %i );
		//if(%sender.team != 0)
	      chatMessageClient( %obj, %sender, %sender.voiceTag, %sender.voicePitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10 );
		//else
		//{
			// message sender is an observer -- only send message to other observers
		//	if(%obj.team == %sender.team || %obj.isAdmin || %obj.isSuperAdmin)
		//      chatMessageClient( %obj, %sender, %sender.voiceTag, %sender.voicePitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10 );
		//}
	}
}

function cannedChatMessageAll( %sender, %msgString, %name, %string, %keys )
{
   if ( ( %msgString $= "" ) || spamAlert( %sender ) )
      return;

   %count = ClientGroup.getCount();
   for ( %i = 0; %i < %count; %i++ )
      cannedChatMessageClient( ClientGroup.getObject(%i), %sender, %msgString, %name, %string, %keys );
}

//---------------------------------------------------------------------------
function messageClient(%client, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13)
{
   commandToClient(%client, 'ServerMessage', %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13);
}

function messageTeam(%team, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13)
{
   %count = ClientGroup.getCount();
   for(%cl= 0; %cl < %count; %cl++)
   {
      %recipient = ClientGroup.getObject(%cl);
	  if(%recipient.team == %team)
	      messageClient(%recipient, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13);
   }
}

function messageTeamExcept(%client, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13)
{
   %team = %client.team;
   %count = ClientGroup.getCount();
   for(%cl= 0; %cl < %count; %cl++)
   {
      %recipient = ClientGroup.getObject(%cl);
	  if((%recipient.team == %team) && (%recipient != %client))
	      messageClient(%recipient, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13);
   }
}

function messageAll(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13)
{
   %count = ClientGroup.getCount();
   for(%cl = 0; %cl < %count; %cl++)
   {
      %client = ClientGroup.getObject(%cl);
      messageClient(%client, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13);
   }
}

function messageAllExcept(%client, %team, %msgtype, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13)
{  
   //can exclude a client, a team or both. A -1 value in either field will ignore that exclusion, so
   //messageAllExcept(-1, -1, $Mesblah, 'Blah!'); will message everyone (since there shouldn't be a client -1 or client on team -1).
   %count = ClientGroup.getCount();
   for(%cl= 0; %cl < %count; %cl++)
   {
      %recipient = ClientGroup.getObject(%cl);
      if((%recipient != %client) && (%recipient.team != %team))
         messageClient(%recipient, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11, %a12, %a13);
   }
}

//---------------------------------------------------------------------------
// functions to support repair messaging
//---------------------------------------------------------------------------
function clientCmdTeamRepairMessage(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6)
{
   if(!$pref::ignoreTeamRepairMessages)
      clientCmdServerMessage(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6);
}

function teamRepairMessage(%client, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6)
{
   %team = %client.team;

   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
      %recipient = ClientGroup.getObject(%cl);
      if((%recipient.team == %team) && (%recipient != %client))
         commandToClient(%client, 'TeamRepairMessage', %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6);
   }
}
